%% IBN due to capacitor mismatch in cascade SDMs
clear; clc;

OSR=[12 16 24 32 48 64];
sigma_cap=logspace(-2,1,20); % Capacitor mismatch in %

e_me=3*sigma_cap/100; % Errors in the integrator weights are considered as 3*sigma_cap
syms z

for j=1:length(OSR);
    for i=1:length(sigma_cap);
        eg=e_me(i);
        g1=0.25; g1p=0.25; g2=1; g2p=0.5; g3=1; g3p=0.5; g3pp=0.5; g4=1; g4p=0.5; g4pp=0.5;
        d0=1; d1=2; d2=0; d3=2;

        IBN1_me_211_approx=1/12*pi^4/(5*OSR(j)^5)*(abs(eg)+abs(eg))^2; 
        IBN2_me_211_approx=1/12*d1^2*pi^6/(7*OSR(j)^7)*(abs(eg))^2;
        IBN3_me_211_approx=1/12*d3^2*pi^8/(9*OSR(j)^9)*(1+abs(eg))^2;
        IBN_me_211_approx(i,j)=IBN1_me_211_approx+IBN2_me_211_approx+IBN3_me_211_approx;

        % Non-approximated NTFs
        eg1=+1;   %
        eg1p=-1;  % 
        eg2=-1;   %
        eg2p=+1;  %
        eg3=+1;   % 
        eg3p=+1;  % 
        eg3pp=+1; % 
        eg4=+1;   % 
        eg4p=+1;  % 
        eg4pp=+1; % 

        NTF1_me_211=(g1*g2*z^2*(((g3p - g1*g2*g3)*(z - 1)^2)/(g1*g2*g3) + 1))/(g1*g2 - (2*g1*g2*z*(eg*eg2p + 1))/(z - 1) + (g1*g2*z^2*(eg*eg1p + 1)*(eg*eg2 + 1))/(z - 1)^2) + (z*((g3*z*(eg*eg3 + 1)*((2*g1^2*g2*g2*z*(eg*eg2p + 1))/(z - 1) - (g1^2*g2*g2*z^2*(eg*eg1p + 1)*(eg*eg2 + 1))/(z - 1)^2))/((z - 1)*(g1*g2 - (2*g1*g2*z*(eg*eg2p + 1))/(z - 1) + (g1*g2*z^2*(eg*eg1p + 1)*(eg*eg2 + 1))/(z - 1)^2)) - (g1*g2*g3p*z*(eg*eg3p + 1))/((z - 1)*(g1*g2 - (2*g1*g2*z*(eg*eg2p + 1))/(z - 1) + (g1*g2*z^2*(eg*eg1p + 1)*(eg*eg2 + 1))/(z - 1)^2)))*(z - 1)^3)/(g1*g2*g3*(eg*eg3pp*z + 1)) + (eg*g2*z^2*(eg4 - eg4p)*(z - 1)^3*(g3p - 2*g3p*z + g3p*z^2 + eg*eg3p*g3p - 2*eg*eg3p*g3p*z + 2*g1*g2*g3*z + eg*eg3p*g3p*z^2 - g1*g2*g3*z^2 + eg*eg1p*g1*g2*g3*z^2 + eg*eg2*g1*g2*g3*z^2 - 2*eg*eg2p*g1*g2*g3*z^2 - eg*eg3*g1*g2*g3*z^2 + 2*eg*eg2p*g1*g2*g3*z + 2*eg*eg3*g1*g2*g3*z + 2*eg^2*eg2p*eg3*g1*g2*g3*z + eg^2*eg1p*eg2*g1*g2*g3*z^2 + eg^2*eg1p*eg3*g1*g2*g3*z^2 + eg^2*eg2*eg3*g1*g2*g3*z^2 - 2*eg^2*eg2p*eg3*g1*g2*g3*z^2 + eg^3*eg1p*eg2*eg3*g1*g2*g3*z^2))/(g1*g2*g3*(eg*eg3pp*z + 1)*(eg*eg4pp*z + 1)*(g2 - 2*g2*z + 2*g2*z + g2*z^2 - g2*z^2 + 2*eg*eg2p*g2*z + eg*eg1p*g2*z^2 + eg*eg2*g2*z^2 - 2*eg*eg2p*g2*z^2 + eg^2*eg1p*eg2*g2*z^2));
        NTF2_me_211=(eg*g3pp*z*(z - 1)^3*(eg4p + eg3pp*z + eg4*z - eg4p*z - eg4pp*z + eg*eg3pp*eg4*z))/(g1*g2*g3*(eg*eg3pp*z + 1)*(eg*eg4pp*z + 1));
        NTF3_me_211=(g4pp*(z - 1)^4)/(g1*g2*g3*g4*(-eg*eg4pp*z + 1));
        IBN1_me_211=num_eval_IBN_sum(NTF1_me_211,1,OSR(j),1e3);
        IBN2_me_211=num_eval_IBN_sum(NTF2_me_211,1,OSR(j),1e3);
        IBN3_me_211=num_eval_IBN_sum(NTF3_me_211,1,OSR(j),1e3);
        IBN_me_211(i,j)=IBN1_me_211+IBN2_me_211+IBN3_me_211;
    end
end

clf;
semilogx(sigma_cap,10*log10(IBN_me_211_approx),'sb-','MarkerSize',8); hold on; grid on;
%semilogx(sigma_cap,10*log10(IBN_me_211),'b--','MarkerSize',8);
%legend('Approx. equation','Non-approx NTFs','Location','SouthEast');
axis([min(sigma_cap) max(sigma_cap) -140 -55]);
xlabel('Capacitor mismatch (%)'); ylabel('IBN (dB)');
x_txt=0.0115;
text(x_txt,-68,'OSR = 12');
text(x_txt,-79,'OSR = 16');
text(x_txt,-95,'OSR = 24');
text(x_txt,-106,'OSR = 32');
text(x_txt,-122,'OSR = 48');
text(x_txt,-133,'OSR = 64');