%% IBN due to capacitor mismatch in single-loop SDMs
clear; clc;

OSR=[32 64];
sigma_cap=logspace(-2,1,20); % Capacitor mismatch in %

e_me=3*sigma_cap/100; % Errors in the integrator weights are considered as 3*sigma_cap
syms z

for j=1:length(OSR);
    for i=1:length(sigma_cap);
        eg=e_me(i);
        IBN_me_2_approx(i,j)=1/12*(1+eg)^2*(1+eg)^2*pi^4/(5*OSR(j)^5);
        IBN_me_3_approx(i,j)=1/12*(1+eg)^2*(1+eg)^2*(1+eg)^2*pi^6/(7*OSR(j)^7);
        % Non-approximated NTF
        eg1p=-1; eg2=-1; eg2p=+1;
        NTF_me_2=1/((z^2*(eg*eg1p + 1)*(eg*eg2 + 1))/(z - 1)^2 - (2*z*(eg*eg2p + 1))/(z - 1) + 1); % From TF_me_2.m - z means z^(-1)
        IBN_me_2(i,j)=num_eval_IBN_sum(NTF_me_2,1,OSR(j),1e3);
        eg1p=-1; eg2=-1; eg2p=-1; eg3=-1; eg3p=+1;
        NTF_me_3=-1/((3*z*(eg*eg3p + 1))/(z - 1) - (3*z^2*(eg*eg2p + 1)*(eg*eg3 + 1))/(z - 1)^2 + (z^3*(eg*eg1p + 1)*(eg*eg2 + 1)*(eg*eg3 + 1))/(z - 1)^3 - 1);
        IBN_me_3(i,j)=num_eval_IBN_sum(NTF_me_3,1,OSR(j),1e3);
    end
end

clf;
semilogx(sigma_cap,10*log10(IBN_me_2_approx(:,1)),'sk-','MarkerSize',8); hold on; grid on;
semilogx(sigma_cap,10*log10(IBN_me_2_approx(:,2)),'+k-','MarkerSize',8);

semilogx(sigma_cap,10*log10(IBN_me_3_approx(:,1)),'ob-','MarkerSize',8);
semilogx(sigma_cap,10*log10(IBN_me_3_approx(:,2)),'xb-','MarkerSize',8);

% semilogx(sigma_cap,10*log10(IBN_me_2(:,1)),'k--','MarkerSize',8);
% semilogx(sigma_cap,10*log10(IBN_me_2(:,2)),'k--','MarkerSize',8);
% 
% semilogx(sigma_cap,10*log10(IBN_me_3(:,1)),'b--','MarkerSize',8);
% semilogx(sigma_cap,10*log10(IBN_me_3(:,2)),'b--','MarkerSize',8);

legend('2nd order, OSR=16','2nd order, OSR=32','3rd order, OSR=16','3rd order, OSR=32','Location','SouthWest');
axis([min(sigma_cap) max(sigma_cap) -120 -65]);
xlabel('Capacitor mismatch (%)'); ylabel('IBN (dB)');