clc;
%% SDM general parameters
L=4; OSR=64; FS=1; Vref=FS/2;
fs=80e6; N=65536;
Ain=0.5*Vref;  fin=1/3*fs/(2*OSR)*0.9897;

%% INTEGRATOR PARAMETERS
sweep=0.1:0.2:3.1; % Calculated to vary GBinteg of each integrator from 0.1fs to 3.1fs in steps of 0.2fs
for j=1:length(sweep)
scaling=sweep(j); 

% scaling=30; %3.1;
gm1=0.88e-3*scaling; % Calculated to vary GBinteg of each integrator from 0.1fs to 3fs in steps of 0.2fs
Io1=50e-3;
Cu1=1e-12;   Cl1=0.5e-12; Cp1=0.1e-12;

gm2=1.09e-3*scaling; % Calculated to vary GBinteg of each integrator from 0.1fs to 3fs in steps of 0.2fs
Io2=50e-3;  
Cu2=0.5e-12; Cl2=0.5e-12; Cp2=0.1e-12;

gm3=1.09e-3*scaling; % Calculated to vary GBinteg of each integrator from 0.1fs to 3fs in steps of 0.2fs
Io3=50e-3;  
Cu3=0.5e-12; Cl3=0.5e-12; Cp3=0.1e-12;

gm4=1.09e-3*scaling; % Calculated to vary GBinteg of each integrator from 0.1fs to 3fs in steps of 0.2fs
Io4=50e-3;  
Cu4=0.5e-12; Cl4=0.5e-12; Cp4=0.1e-12;

Cs1=Cu1;  Ci1 =4*Cs1;             %% INTEG 1: gain 0.25
Cs21=Cu2; Cs22=Cs21; Ci2=2*Cs21;  %% INTEG 2: gain 1, 0.5
Cs31=Cu3; Cs32=Cs31; Ci3=2*Cs31;  %% INTEG 3: gain 1, 0.5, 0.5
Cs41=Cu4; Cs42=Cs41; Ci4=2*Cs41;  %% INTEG 4: gain 1, 0.5, 0.5

OS=5; Ron=1;

%% QUANTIZER PARAMETERS
ts=1/fs;
Vq=Vref;

%% SETTLING CALCULATIONS
Ceqi=zeros(1,L); Ceqs=zeros(1,L);
GBi=zeros(1,L);  GBs=zeros(1,L);
SRi=zeros(1,L);  SRs=zeros(1,L);

Ceqi(1)=Cp1+Cs1+Cl1*(1+(Cp1+Cs1)/Ci1);
Ceqs(1)=Cp1+(Cl1+Cs21+Cs22)*(1+Cp1/Ci1);

Ceqi(2)=Cp2+(Cs21+Cs22)+Cl2*(1+(Cp2+(Cs21+Cs22))/Ci2);
Ceqs(2)=Cp2+(Cl2+Cs31+Cs32)*(1+Cp2/Ci2);

Ceqi(3)=Cp3+(Cs31+Cs32)+Cl3*(1+(Cp3+(Cs31+Cs32))/Ci3);
Ceqs(3)=Cp3+(Cl3+Cs41+Cs42)*(1+Cp3/Ci3);

Ceqi(4)=Cp4+(Cs41+Cs42)+Cl4*(1+(Cp4+(Cs41+Cs42))/Ci4);
Ceqs(4)=Cp4+Cl4*(1+Cp4/Ci4);

[GBi(1),GBs(1),SRi(1),SRs(1)]=settling_param(Cs1,Ci1,Cp1,Cl1,Ceqi(1),Ceqs(1),gm1,Io1);
[GBi(2),GBs(2),SRi(2),SRs(2)]=settling_param([Cs21 Cs22],Ci2,Cp2,Cl2,Ceqi(2),Ceqs(2),gm2,Io2);
[GBi(3),GBs(3),SRi(3),SRs(3)]=settling_param([Cs31 Cs32],Ci3,Cp3,Cl3,Ceqi(3),Ceqs(3),gm3,Io3);
[GBi(4),GBs(4),SRi(4),SRs(4)]=settling_param([Cs41 Cs42],Ci4,Cp4,Cl4,Ceqi(4),Ceqs(4),gm4,Io4);

fprintf('\n');
for k=1:L
fprintf('\n ******************** INTEG #%i *******************', k);
fprintf('\n Ceq,int = %.2f pF      \t*  Ceq,sam = %.2f pF     ', Ceqi(k)*1e12,Ceqs(k)*1e12);
fprintf('\n GBinteg = %.2f MHz     \t*  GBsampl = %.2f MHz    ', GBi(k)*1e-6,GBs(k)*1e-6);
fprintf('\n         = %.2f fs      \t*          = %.2f fs     ', GBi(k)/fs,GBs(k)/fs);
fprintf('\n SRinteg = %.2f V/us    \t*  SRsampl = %.2f V/us   ', SRi(k)*1e-6,SRs(k)*1e-6);
fprintf('\n         = %.2f Vref*fs \t*          = %.2f Vref*fs', SRi(k)/(Vref*fs),SRs(k)/(Vref*fs));
fprintf('\n');
end
fprintf('\n');

sim('MASH211_settling.mdl');

[pxx,f,pxx_bin,fbin]=spect(y4,fs);

figure;
semilogx(f,10*log10(pxx_bin)); hold on; grid on;
xlabel('Frequency (Hz)'); ylabel('PSD x bin (dB)');

indsig=max(find(f<=fin));
nsig=100;
Psig=sum(pxx_bin(indsig-nsig:indsig+nsig));
semilogx(f(indsig-nsig:indsig+nsig),10*log10(pxx_bin(indsig-nsig:indsig+nsig)),'r');

BW=fs/(2*OSR);
indBW=max(find(f<=BW));
PBW=sum(pxx_bin(1:indBW));
IBN(j)=10*log10(PBW-Psig)

end