clear;
%% SDM general parameters
L=2; OSR=64; FS=1; Vref=FS/2;
fs=80e6; N=65536;
Ain=0.1*Vref;  fin=1/3*fs/(2*OSR)*0.9897;

%% White noise parameters
t=0:1/fs:(N-1)*1/fs; % Time vector generation
BW_noise=fs/(2*OSR);
noise_rms=1e-4; % Eq. output-referred noise in the pre-amp expressed in Vrms (integrated in the BW_noise bandwidth)
noise=gen_white_noise_RRF(noise_rms,BW_noise,fs,N);

%% INTEGRATOR PARAMETERS
sweep=0.1:0.2:3.1; % Calculated to vary GBinteg of each integrator from 0.1fs to 3.1fs in steps of 0.2fs
for j=1:4 %:length(sweep)
scaling=sweep(j); 
% scaling=30; %3.1;

gm1=0.945e-3*scaling; % Calculated to vary GBinteg of each integrator from 0.1fs to 3fs in steps of 0.2fs
Io1=50e-3;
Cu1=1e-12;   Cl1=0.5e-12; Cp1=0.1e-12;

gm2=0.605e-3*scaling; % Calculated to vary GBinteg of each integrator from 0.1fs to 3fs in steps of 0.2fs
Io2=50e-3;  
Cu2=0.25e-12; Cl2=0.5e-12; Cp2=0.1e-12;

% a1=0.5; a2=0.5;
Cs1=Cu1; Ci1=2*Cu1;
Cs2=Cu2; Ci2=2*Cu2;

OS=5; Ron=1;

%% QUANTIZER PARAMETERS
ts=1/fs;
Vq=Vref;

%% SETTLING CALCULATIONS
Ceqi=zeros(1,L); Ceqs=zeros(1,L);
GBi=zeros(1,L);  GBs=zeros(1,L);
SRi=zeros(1,L);  SRs=zeros(1,L);

Ceqi(1)=Cp1+Cs1+Cl1*(1+(Cp1+Cs1)/Ci1);
Ceqs(1)=Cp1+(Cl1+Cs2)*(1+Cp1/Ci1);

Ceqi(2)=Cp2+Cs2+Cl2*(1+(Cp2+Cs2)/Ci2);
Ceqs(2)=Cp2+Cl2*(1+Cp2/Ci2);

[GBi(1),GBs(1),SRi(1),SRs(1)]=settling_param(Cs1,Ci1,Cp1,Cl1,Ceqi(1),Ceqs(1),gm1,Io1);
[GBi(2),GBs(2),SRi(2),SRs(2)]=settling_param(Cs2,Ci2,Cp2,Cl2,Ceqi(2),Ceqs(2),gm2,Io2);

fprintf('\n');
for k=1:L
fprintf('\n ******************** INTEG #%i *******************', k);
fprintf('\n Ceq,int = %.2f pF      \t*  Ceq,sam = %.2f pF     ', Ceqi(k)*1e12,Ceqs(k)*1e12);
fprintf('\n GBinteg = %.2f MHz     \t*  GBsampl = %.2f MHz    ', GBi(k)*1e-6,GBs(k)*1e-6);
fprintf('\n         = %.2f fs      \t*          = %.2f fs     ', GBi(k)/fs,GBs(k)/fs);
fprintf('\n SRinteg = %.2f V/us    \t*  SRsampl = %.2f V/us   ', SRi(k)*1e-6,SRs(k)*1e-6);
fprintf('\n         = %.2f Vref*fs \t*          = %.2f Vref*fs', SRi(k)/(Vref*fs),SRs(k)/(Vref*fs));
fprintf('\n');
end
fprintf('\n');

sim('SL2_settling.mdl');

[pxx,f,pxx_bin,fbin]=spect(y,fs);

figure;
semilogx(f,10*log10(pxx_bin)); hold on; grid on;
xlabel('Frequency (Hz)'); ylabel('PSD x bin (dB)');

indsig=max(find(f<=fin));
nsig=100;
Psig=sum(pxx_bin(indsig-nsig:indsig+nsig));
semilogx(f(indsig-nsig:indsig+nsig),10*log10(pxx_bin(indsig-nsig:indsig+nsig)),'r');

BW=fs/(2*OSR);
indBW=max(find(f<=BW));
PBW=sum(pxx_bin(1:indBW));
IBN(j)=10*log10(PBW-Psig)

end