%% ESTIMATION OF IN-BAND ERROR CONTRIBUTIONS FROM CLOSED-FORM EQUATIONS
%% 2-1-1 SDM
% RRF - 12/07/12
%clear all; clc;

%% INPUT PARAMETERS
L=4; fs=100e6; Bw=4e6; FS=2; % FS=2*Vref=2*1, where Vref stands for the differential reference voltage
B1=1; B2=1; B3=3; % DAC errors are computed assuming multibit quant. only in the last stage
Tmax=27; % Max. temperature in C

g1=0.25; g1p=0.25; g2=1; g2p=0.5; g3=1; g3p=0.5; g3pp=0.5; g4=1; g4p=0.5; g4pp=0.5;
d0=1; d1=2; d2=0; d3=2;

Av_dB=65;
sigma_cap=0.1; % Capacitor matching in %
sigma_res=0.1; % Resistor matching in % (Equivalent to INL_DAC=0.07%FS)

Cs=0.4e-12; Ci=1/g1p*Cs;
Cp=0.1e-12; Cl=0.5e-12; 
gm=1.5e-3;
OSR=fs/(2*Bw);

k=1.38e-23; T=Tmax+273;
Sth_amp=(6e-9)^2; % Equivalente input amplifier noise expressed in V^2/Hz

%% MODULATOR COEFFICIENTS
a1=g1p*g2/g2p;
a2=g2p;
kq=2/a2; % Assuming sibgle-bit quant in the 1st SD stage
a3=g3pp;
a4=g4pp;
b1=g3p/(g1p*g2*g3);
b2=g4p/(g3pp*g4);
c1=g1p*g2*g3/g3pp;
c2=g3pp*g4/g4pp;
Coeffs=[a1 a2 a3 a4 b1 c1 b2 c2];

%% QUANTIZATION ERROR CONTRIBUTION
Delta1=FS/(2^B1-1);
Delta2=FS/(2^B2-1);
Delta3=FS/(2^B3-1);
sigmaQ1=Delta1^2/12;
sigmaQ2=Delta2^2/12;
sigmaQ3=Delta3^2/12;

P_Q=sigmaQ3*1/(c1*c2)^2*pi^8/(9*OSR^9);

%% DAC ERROR CONTRIBUTION
%sigma_DAC=(FS^2*(INL_DAC/100)^2)/2;  % DAC-induced error power with INL referred to FS integral non-linearity
sigma_DAC=(1/(2*sqrt(2*B3))*3*sigma_res/100)^2;
P_DAC=1/(c1*c2)^2*sigma_DAC*pi^6/(7*OSR^7);

%% INTEGRATOR LEAKAGES CONTRIBUTION
Av=10^(Av_dB/20);
P_Av=sigmaQ1*1/(kq*a1*a2)^2*4*pi^2/(3*Av^2*OSR^3)+...
    sigmaQ2*1/(c1)^2*pi^4/(5*Av^2*OSR^5)+sigmaQ3*1/(c1*c2)^2*pi^6/(7*Av^2*OSR^7);

%% CAPACITOR MISMATCH
eg=3*sigma_cap/100;
P_eg=sigmaQ1*1/(kq*a1*a2)^2*pi^4/(5*OSR^5)*(eg+eg)^2+...
    sigmaQ2*1/(c1)^2*pi^6/(7*OSR^7)*(eg)^2;

%% SETTLING ERROR - LINEAR ERROR APPROXIMATION
Ceqi=Cp+Cs+Cl*(1+(Cp+Cs)/Ci);
GBi=gm/Ceqi; % Expressed in rad/s
e_st=exp(-GBi/(2*fs));
P_st=sigmaQ1*1/(kq*a1*a2)^2*pi^4/(5*OSR^5)*(e_st+e_st)^2+...
    sigmaQ2*1/(c1)^2*pi^6/(7*OSR^7)*(e_st)^2;

%% kT/C NOISE CONTRIBUTION
P_kTC=4*k*T/Cs*1/OSR;

%% AMPLIFIER THERMAL NOISE CONTRIBUTION
P_amp=Sth_amp*GBi/4*1/OSR;

%% TOTAL NOISE CONTRIBUTIONS
IBN=P_Q+P_DAC+P_Av+P_eg+P_st+P_kTC+P_amp;
DR=(FS/2)^2/(2*IBN);
DR_id=(FS/2)^2/(2*P_Q);

fprintf('\n **************************************************')
fprintf('\n **  IBN ESTIMATIONS AT THE MODULATOR OUTPUT     **')
fprintf('\n **************************************************')
fprintf('\n')
fprintf('\n QUANTIZATION NOISE    =   %.1f dB   =  %.1f uVrms', 10*log10(P_Q), sqrt(P_Q)*1e6)
fprintf('\n --------------------------------------------------')
fprintf('\n TOTAL LEAKAGES        =   %.1f dB   =  %.1f uVrms', 10*log10(P_DAC+P_Av+P_eg), sqrt(P_DAC+P_Av+P_eg)*1e6)
fprintf('\n     DAC Nonlin        =   %.1f dB   =  %.1f uVrms', 10*log10(P_DAC), sqrt(P_DAC)*1e6)
fprintf('\n     Integ. Leakage    =   %.1f dB   =  %.1f uVrms', 10*log10(P_Av), sqrt(P_Av)*1e6)
fprintf('\n     Cap. Mismatch     =   %.1f dB   =  %.1f uVrms', 10*log10(P_eg), sqrt(P_eg)*1e6)
fprintf('\n --------------------------------------------------')
fprintf('\n Quant. + Leakages     =   %.1f dB   =  %.1f uVrms', 10*log10(P_Q+P_DAC+P_Av+P_eg), sqrt(P_Q+P_DAC+P_Av+P_eg)*1e6)
fprintf('\n --------------------------------------------------')
fprintf('\n SETTLING ERROR (lin)  =   %.1f dB   =  %.1f uVrms', 10*log10(P_st), sqrt(P_st)*1e6)
fprintf('\n     Eq. load (integ)  =   %.1f pF', Ceqi*1e12)
fprintf('\n     GB (integ)        =   %.1f MHz  =  %.1f fs', GBi/(2*pi)*1e-6, GBi/(2*pi)/fs)
fprintf('\n     Sampling freq.    =   %.1f MHz', fs*1e-6)
fprintf('\n --------------------------------------------------')
fprintf('\n THERMAL NOISE (input) =   %.1f dB   =  %.1f uVrms', 10*log10(P_kTC+P_amp), sqrt(P_kTC+P_amp)*1e6)
fprintf('\n     kT/C Noise        =    %.1f dB  =  %.1f uVrms', 10*log10(P_kTC), sqrt(P_kTC)*1e6)
fprintf('\n     Amplif. Noise     =    %.1f dB  =  %.1f uVrms', 10*log10(P_amp), sqrt(P_amp)*1e6)
fprintf('\n')
fprintf('\n -------------------------------')
fprintf('\n TOTAL IBN  =  %.1f dB =  %.1f uVrms', 10*log10(IBN), sqrt(IBN)*1e6)
fprintf('\n   DR       =   %.1f dB =  %.1f bit',   10*log10(DR), (10*log10(DR)-1.76)/6.02)
fprintf('\n   DR ideal =   %.1f dB =  %.1f bit',   10*log10(DR_id), (10*log10(DR_id)-1.76)/6.02)
fprintf('\n\n')