%% ESTIMATION OF IN-BAND ERROR CONTRIBUTIONS FROM CLOSED-FORM EQUATIONS
%% 2-1-1 SDM
% RRF - 12/07/12
clear all; clc; clf;

%% INPUT PARAMETERS
L=4; fs=100e6; Bw=4e6; FS=2; % FS=2*Vref=1, where Vref stands for the differential reference voltage
B1=1; B2=1; B3=3; % DAC errors are computed assuming multibit quant. only in the last stage
Tmax=27; % Max. temperature in C

g1=0.25; g1p=0.25; g2=1; g2p=0.5; g3=1; g3p=0.5; g3pp=0.5; g4=1; g4p=0.5; g4pp=0.5;
d0=1; d1=2; d2=0; d3=2;

Av_dB=65;
sigma_cap=0.1; % Capacitor matching in %
sigma_res=0.1; % Resistor matching in % (Equivalent to INL_DAC=0.07%FS)

%OSR=fs/(2*Bw);
% B3=3;
OSR_sweep=10:2:24;
B3=1:1:5;

%% MODULATOR COEFFICIENTS
a1=g1p*g2/g2p;
a2=g2p;
kq=2/a2; % Assuming sibgle-bit quant in the 1st SD stage
a3=g3pp;
a4=g4pp;
b1=g3p/(g1p*g2*g3);
b2=g4p/(g3pp*g4);
c1=g1p*g2*g3/g3pp;
c2=g3pp*g4/g4pp;
Coeffs=[a1 a2 a3 a4 b1 c1 b2 c2];

for i=1:length(OSR_sweep)
    
    OSR=OSR_sweep(i);
    
    %% QUANTIZATION ERROR CONTRIBUTION
    Delta1=FS/(2^B1-1);
    Delta2=FS/(2^B2-1);
    Delta3=FS./(2.^B3-1);
    sigmaQ1=Delta1^2/12;
    sigmaQ2=Delta2^2/12;
    sigmaQ3=Delta3.^2/12;
    
    P_Q=sigmaQ3*1/(c1*c2)^2*pi^8/(9*OSR^9);
    
    %% DAC ERROR CONTRIBUTION
    %sigma_DAC=(FS^2*(INL_DAC/100)^2)/2;  % DAC-induced error power with INL referred to FS integral non-linearity
    sigma_DAC=(1./(2*sqrt(2*B3))*3*sigma_res/100).^2;
    P_DAC=1/(c1*c2)^2*sigma_DAC*pi^6/(7*OSR^7);
    
    %% INTEGRATOR LEAKAGES CONTRIBUTION
    Av=10^(Av_dB/20);
    P_Av=sigmaQ1*1/(kq*a1*a2)^2*4*pi^2/(3*Av^2*OSR^3)+...
        sigmaQ2*1/(c1)^2*pi^4/(5*Av^2*OSR^5)+sigmaQ3*1/(c1*c2)^2*pi^6/(7*Av^2*OSR^7);
    
    %% CAPACITOR MISMATCH
    eg=3*sigma_cap/100;
    P_eg=sigmaQ1*1/(kq*a1*a2)^2*pi^4/(5*OSR^5)*(eg+eg)^2+...
        sigmaQ2*1/(c1)^2*pi^6/(7*OSR^7)*(eg)^2;
    
    %% TOTAL NOISE CONTRIBUTIONS
    IBN=P_Q+P_DAC+P_Av+P_eg;
    DR=(FS/2)^2./(2*IBN);
    DR_id=(FS/2)^2./(2*P_Q);
    ENOB=(10*log10(DR)-1.76)/6.02;
    ENOB_id=(10*log10(DR_id)-1.76)/6.02;
    
    plot(B3,ENOB,'b--s'); hold on; grid on;
    %plot(B3,ENOB_id,'b-.');
    xlabel('Resolution of the last-stage quantizer, B_3 (bit)')
    ylabel('Modulator ENOB (bit)')
    %legend('OSR=10,12,14,...,24',4)

end

axis([1 5 8 16]);

x_txt=2.2;
text(x_txt,11.1,'OSR = 10');
text(x_txt,12.2,'OSR = 12');
text(x_txt,13.1,'OSR = 14');
text(x_txt,13.8,'OSR = 16');
text(x_txt,14.4,'OSR = 18');
text(x_txt,14.8,'OSR = 20');
text(x_txt,15.2,'OSR = 22');
text(x_txt,15.6,'OSR = 24');