#define S_FUNCTION_NAME pl_integrator
#define  S_FUNCTION_LEVEL 2

#include "simstruc.h"
#include <math.h>

#define gm(S) ssGetSFcnParam(S,0) /* Nominal transconductance */
#define C(S)  ssGetSFcnParam(S,1) /* Nominal capacitance */ 
#define et(S) ssGetSFcnParam(S,2) /* Time constant error (%) */
#define Av(S) ssGetSFcnParam(S,3) /* Finite DC gain */
#define ub(S) ssGetSFcnParam(S,4) /* Output swing limitation (max) */
#define lb(S) ssGetSFcnParam(S,5) /* Output swing limitation (min) */
#define iub(S) ssGetSFcnParam(S,6)/* Input swing limitation (max) */
#define ilb(S) ssGetSFcnParam(S,7)/* Input swing limitation (min) */
#define gm1(S) ssGetSFcnParam(S,8)/* First-order transconductance non-linearity */
#define gm2(S) ssGetSFcnParam(S,9)/* Second-order transconductance non-linearity */
#define selector_param(S) ssGetSFcnParam(S,10)
#define N_PARAMS 11 

static void mdlInitializeSizes (SimStruct *S)
{
ssSetNumSFcnParams(S, N_PARAMS);
    ssSetNumContStates(S, 1);
    ssSetNumDiscStates(S, 0);
    ssSetNumInputPorts(S, 3);
    ssSetInputPortWidth(S, 0, 1);
    ssSetInputPortWidth(S, 1, 1);
    ssSetInputPortWidth(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 0, 0);
    ssSetInputPortDirectFeedThrough(S, 1, 0);
    ssSetInputPortDirectFeedThrough(S, 2, 0);
    ssSetNumOutputPorts(S, 1);
    ssSetOutputPortWidth(S, 0, 1);
    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 0);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);
    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}

/* Continuous-time nature of the circuit is defined here */

static void mdlInitializeSampleTimes(SimStruct *S)
{
	ssSetSampleTime( S, 0, CONTINUOUS_SAMPLE_TIME);
	ssSetOffsetTime ( S, 0, 0.0);
}

#define MDL_INITIALIZE_CONDITIONS

static void mdlInitializeConditions (SimStruct *S)
{
	real_T *x0 = ssGetContStates(S) ;
   	x0[0] = 0; /* State variable */
}

static void mdlOutputs (SimStruct *S, int_T tid)
{
	real_T	*y	= ssGetOutputPortRealSignal (S, 0);
	real_T	*x	= ssGetContStates (S);
	real_T ub = *mxGetPr(ub(S));
    real_T lb = *mxGetPr(lb(S));

	/*Checking output saturation*/
    
    if(x[0]<=lb)
        {x[0]=lb;}
    if(x[0]>=ub)
        {x[0]=ub;}
	
	y[0] = x[0] ;
}

#define MDL_DERIVATIVES
/* Function : mdlDerivatives =============================================
 *		xdot = Ax + Bu  */

static void mdlDerivatives(SimStruct *S)
{
    real_T *dx	= ssGetdX(S);
	real_T *x = ssGetContStates (S);
	real_T ub = *mxGetPr(ub(S));
    real_T lb = *mxGetPr(lb(S));
	real_T iub = *mxGetPr(iub(S));
    real_T ilb = *mxGetPr(ilb(S));	
	real_T gm1 = *mxGetPr(gm1(S));
    real_T gm2 = *mxGetPr(gm2(S));	    
	real_T C = *mxGetPr(C(S));
    real_T gm = *mxGetPr(gm(S));	    
	real_T et = *mxGetPr(et(S));
    real_T Av = *mxGetPr(Av(S));	    
	real_T  selector = *mxGetPr(selector_param(S));
    InputRealPtrsType uPtrs0 = ssGetInputPortRealSignalPtrs(S,0);
    InputRealPtrsType uPtrs1 = ssGetInputPortRealSignalPtrs(S,1);
    InputRealPtrsType uPtrs2 = ssGetInputPortRealSignalPtrs(S,2);
    real_T vin = *uPtrs0[0]; /* Input voltage */
    real_T iin = *uPtrs1[0]; /* Input current (from the modulator feedback DAC */
    real_T noise = *uPtrs2[0];/* Input noise source */
    real_T input; 
    real_T go; /* Output conductance */
    C=C*(1+et); /* Integration capacitance modification due to time-constant error */
    go=gm/Av;   /* Finite DC gain: Av=gm/go */

	if (selector ==1) /* Input voltage without noise contribution */
		{input=vin;}
	else
		{input=vin+noise;} /* Input voltage considering noise contribution */
		
    /*Non lineal transconductance*/
    
    input=input+gm1*pow(input,2)+gm2*pow(input,3); 

    /*Output swing limitation */
    
    if(x[0]<=lb)
        {x[0]=lb;}
    if(x[0]>=ub)
        {x[0]=ub;}
    
    /*Input swing limitation */
    
    if(input<=ilb)
        {input=ilb;}
    if(input>=iub)
        {input=iub;}
    
    dx[0]=-go/C*x[0]+gm/C*input+1/C*iin; /* Differential equation to evaluate transient response */
                                           /* 1-pole case */

}

static void mdlTerminate(SimStruct *S)
{
}

# ifdef	MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cd_sfun.h"
#endif
