#define S_FUNCTION_NAME MultiLevelQuantizer
#define S_FUNCTION_LEVEL 2

#include "tmwtypes.h"
#include "simstruc.h"
#include <math.h>
#include <string.h>
#define Vhigh(S) ssGetSFcnParam(S,0) /* Output saturation voltage (max) */
#define Vlow(S) ssGetSFcnParam(S,1)	 /* Output saturation voltage (min) */
#define phi(S) ssGetSFcnParam(S,2)   /* Sampling clock phase (phi1, phi2) */
#define ts(S) ssGetSFcnParam(S,3)    /* Sampling time */
#define n_levels(S) ssGetSFcnParam(S,4) /* Number of levels */
#define Xfs(S) ssGetSFcnParam(S,5)      /* Input full-scale range */
#define inl_param(S) ssGetSFcnParam(S,6) /* Integral Non-Linearity (INL) */
#define epsilong_param(S) ssGetSFcnParam(S,7) /* Offset parameter (epsilon_g) */
#define epsilonoff_param(S) ssGetSFcnParam(S,8) /* Offset parameter (epsilon_off) */
#define NPARAM 9

static void mdlInitializeSizes(SimStruct *S)
{...}

static void mdlInitializeSampleTimes(SimStruct *S) /* Clock-phase scheme */
{
   double PHI, TS;
   TS = *mxGetPr(ts(S));
   PHI = *mxGetPr(phi(S));
   if(PHI == 1){
      ssSetSampleTime(S,0,TS);
      ssSetOffsetTime(S,0,0);}
   else{
      ssSetSampleTime(S,0,TS);
      ssSetOffsetTime(S,0,(PHI-1)/2 * TS);}
}

static void mdlOutputs(SimStruct *S, int_T tid)
{
   InputRealPtrsType uPtrs = ssGetInputPortRealSignalPtrs(S,0);
   real_T *y = ssGetOutputPortRealSignal(S,0);
   int_T  yWidth       = ssGetOutputPortWidth(S,0);
   double U, analog_out, delta, Xlsb, INL, gain, epsilon_g, epsilon_off;
   int  n_levels, n_steps, i, dig_level;
   double in_range, fsr, vin, vhigh, vlow, n, epsilon, A, l1, off, gamma, wa, ya;
   U = *uPtrs[0]; /* Input voltage */
   vhigh = *mxGetPr(Vhigh(S));
   vlow = *mxGetPr(Vlow(S));
   n_levels = *mxGetPr(n_levels(S)); 
   fsr = vhigh - vlow; /* Output Full-Scale (FS) */
   in_range = *mxGetPr(Xfs(S)); /* Input range */
   INL =           *mxGetPr(inl_param(S));
   epsilon_g =     *mxGetPr(epsilong_param(S));
   epsilon_off =   *mxGetPr(epsilonoff_param(S));
   n_steps = n_levels-1; /* Number of quantization steps */
   delta = (fsr/n_steps); /* Quantization step (Delta parameter) */
   Xlsb = (in_range/n_steps); /* XLSB */
   gain = fsr / in_range; /* Quantizer gain */

   /*  Computation of offset, gain and non-linearity */

   if (n_levels<=2)
   {
     epsilon=0;
   }
   else
   { epsilon =(sqrt(27)*INL)/(n_levels-2);}
   A       =(n_levels-1)*Xlsb; /* A_Q parameter */
   l1      =(-fsr/(2*gain))+(Xlsb/2);
   gamma    =1/(1+(Xlsb*epsilon_g)); /* gamma parameter */
   off     =((l1*epsilon_g) - epsilon_off) * Xlsb; /* v_off (offset) voltage */
   wa = gamma*(U+off); /* Computing wa */
   ya = (1-epsilon)*wa + (epsilon/pow(A,2))* pow(wa,3); /* Computing ya */

  /* Ideal Multi-Level ADC operation */
	
   vin = fabs(ya) - Xlsb; /* Input to the ideal quantizer */
   
   if ((n_levels%2)==0) /* Even number of levels (Midrise Quantization)  */
   {	 
     if (fabs(ya) <  Xlsb)
     {
       analog_out = (delta/2)*(ya>=0 ? 1: -1);
     }

     if (( fabs(ya) >= (in_range/2)-(Xlsb/2)))
     {
       analog_out = fsr/2*(ya>0 ? 1: -1);
     }

     if((fabs( ya )<(in_range/2)-(Xlsb/2)) && (fabs(ya)>Xlsb))
     {
       n = floor(vin/Xlsb);
       analog_out = ((delta/2) + (n+1)*delta)*(ya>0 ? 1: -1);
     }
    } 
   else /* Odd number of levels (Midtreat Quantization) */
   {
     analog_out = delta * floor(fabs(ya/delta) + 0.5) * 
                   (ya >= 0.0 ? 1.0 : -1.0);
     if (analog_out>vhigh) analog_out=vhigh;
     if (analog_out<vlow) analog_out=vlow;
	   
   }
   /* Digital output generation*/   
   for (i=0; i<=yWidth; i++)
   {
     dig_level=(int)((analog_out-vlow)/delta);
     if (i==dig_level)   
       y[i]=1;
     else
       y[i]=0;
   }
}
static void mdlTerminate(SimStruct *S)
{}
#ifdef MATLAB_MEX_FILE
#include "simulink.c"
#else
#include "cg_sfun.h"
#endif

