/*  File name  : intfesc1branchavnl.c
 *  Description: Behavioral model of a SC-FE integrator with non-linear finite DC Gain.       
 *  Level-2 S-Function.
 */


#define S_FUNCTION_NAME intfesc1branchavnl
#define S_FUNCTION_LEVEL 2

#include "tmwtypes.h"
#include "simstruc.h"
#include <math.h>
#include<string.h>

#define phi(S)     ssGetSFcnParam(S,0)
#define ts(S)      ssGetSFcnParam(S,1)
#define c1(S)      ssGetSFcnParam(S,2)
#define c2(S)      ssGetSFcnParam(S,3)
#define av(S)      ssGetSFcnParam(S,4)
#define avnl(S)    ssGetSFcnParam(S,5)
#define opspos(S)  ssGetSFcnParam(S,6)
#define opsneg(S)  ssGetSFcnParam(S,7)
#define cp(S)       ssGetSFcnParam(S,8)

#define NPARAM 9


/*  Function:  mdlInitializeSizes
 *  Description: 
 *  To determine the S-function block's characteristics(number of inputs,
 *  outputs, states,etc). 
 */
static void mdlInitializeSizes(SimStruct *S)
{
    
    ssSetNumSFcnParams(S, NPARAM); 
    if (ssGetNumSFcnParams(S)  != NPARAM)
       return;
    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 2)) return;
    ssSetInputPortWidth (S, 0, 1);
    ssSetInputPortWidth (S, 1, 1);
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    
    if (!ssSetNumOutputPorts(S, 1)) return;
    ssSetOutputPortWidth (S, 0, 1);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 5);
    ssSetNumIWork(S, 0);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);
    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
}

/*  Function:   mdlInitializeSampleTimes======================================
 *  Description: 
 *     Initialize the sample times array.
 *     We must include the number of pair "ssSampleTime,ssSetOffsetTime" 
 *     that apears in "ssSetNumSampleTimes" above.
 */

static void mdlInitializeSampleTimes(SimStruct *S)
{
    double TS;
   
    TS = *mxGetPr(ts(S));
    ssSetSampleTime(S, 0, TS/2);
    ssSetOffsetTime(S, 0, 0);
}


static void mdlStart(real_T *x0,SimStruct *S)
{
    real_T *work = ssGetRWork(S);
    work[0]=0;
    work[1]=0;
    work[2]=0;
    work[3]=0;
    work[4]=1000;
}  

/* Function:   mdlOutputs=====================================================
 * Description:
 *      Compute the output. In this function, you compute the outputs of 
 *      your S-function block. Outputs are placed in the output vector(S),
 *      "ssGetOutputPortSignal".
 */

static void mdlOutputs(SimStruct *S, int_T tid)
{
     double U1, U2, PHI, cont, VC1, VC2, VC1ANT, C1, C2, CP, ITER, AV, AVNL1,AVNL2, AVNL3, AVNL4, AVANT, AVNUE, AVVAR, AV_ANT, VO2, VO3, VO4, temp, OPSPOS, OPSNEG;
     double *AVNL;
     InputRealPtrsType uPtrs1 = ssGetInputPortRealSignalPtrs(S,0);
     InputRealPtrsType uPtrs2 = ssGetInputPortRealSignalPtrs(S,1);
     real_T *y = ssGetOutputPortRealSignal(S,0);
     real_T *work = ssGetRWork(S);
     PHI = *mxGetPr(phi(S));
     C1 = *mxGetPr(c1(S));
     C2 = *mxGetPr(c2(S));
     CP = *mxGetPr(cp(S));
     AV = *mxGetPr(av(S));
     AVNL= mxGetPr(avnl(S));
     AVNL1 = AVNL[0];
     AVNL2 = AVNL[1];
     AVNL3 = AVNL[2];
     AVNL4 = AVNL[3];
     OPSPOS = *mxGetPr(opspos(S));
     OPSNEG = *mxGetPr(opsneg(S));
  
     U1=*uPtrs1[0];
     U2=*uPtrs2[0];
     cont=work[0];
     VC1=work[1];
     VC2=work[2];
     VC1ANT=work[3];
     AV_ANT=work[4];
     
     if(PHI==1){
            if(cont==0){
                VC1 = U1;
                AVNUE = AV;
                ITER = 0;
                do{
                    temp = y[0]*(1+1/AVNUE);
                    AVANT = AVNUE;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNUE = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNUE)/AVNUE);
                    ITER = ITER + 1;
                }while(AVVAR>0.01&&ITER<50);
                VC2=temp;
                work[4]=AVNUE;
                cont=cont+1;}
            else{
                VC1ANT = VC1;
                VC1 = U2;
                AVNUE = AV;
                ITER = 0;
                do{
                    temp = ((C1/C2)*(VC1ANT-VC1) + VC2*(1+(1+CP/C2)/AV_ANT))/(1+(1+(C1+CP)/C2)/AVNUE);
                    AVANT = AVNUE;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNUE = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNUE)/AVNUE);
                    ITER = ITER + 1;
                }while(AVVAR>0.01&&ITER<50);
                y[0] = temp;
                if(y[0]>OPSPOS){
                    y[0] = OPSPOS;
                }
                if(y[0]<OPSNEG){
                    y[0] = OPSNEG;
                }
                cont=0;}
     }
     else{
            if(cont==0){
                VC1ANT = VC1;
                VC1 = U2;
                AVNUE = AV;
                ITER = 0;
                do{
                    temp = ((C1/C2)*(VC1ANT-VC1) + VC2*(1+(1+CP/C2)/AV_ANT))/(1+(1+(C1+CP)/C2)/AVNUE);
                    AVANT = AVNUE;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNUE = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNUE)/AVNUE);
                    ITER = ITER + 1;
                }while(AVVAR>0.01&&ITER<50);
                y[0] = temp;
                if(y[0]>OPSPOS){
                    y[0] = OPSPOS;
                }
                if(y[0]<OPSNEG){
                    y[0] = OPSNEG;
                }
                cont=cont+1;}
            else{
                VC1 = U1;
                AVNUE = AV;
                ITER = 0;
                do{
                    temp = y[0]*(1+1/AVNUE);
                    AVANT = AVNUE;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNUE = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNUE)/AVNUE);
                    ITER = ITER + 1;
                }while(AVVAR>0.01&&ITER<50);
                VC2=temp;
                work[4]=AVNUE;
                cont=0;}
     }        
     work[0]=cont;
     work[1]=VC1;
     work[2]=VC2;
     work[3]=VC1ANT;
}


/* Function:   mdlTerminate===================================================
 * mdlTerminate - called when the simulation is terminated.
 *
 * In this function, you should perform any actions that are necessary
 * at the termination of a simulation.  For example, if memory was allocated
 * in mdlInitializeConditions, this is the place to free it.
 */

static void mdlTerminate(SimStruct *S)
{
}

/* At the end of our program we must include this lines.*/
#ifdef  MATLAB_MEX_FILE    /* Is this file being compiled as a MEX-file? */
#include "simulink.c"      /* MEX-file interface mechanism */
#else
#include "cg_sfun.h"       /* Code generation registration function */
#endif


