/* File   :   intfesc1branch_allerrors.c
 *  Description: SC-FE integrator with all the non-ideal effects.       
 */

#define S_FUNCTION_NAME intfesc1branch_allerrors
#define S_FUNCTION_LEVEL 2

#include "tmwtypes.h"
#include "simstruc.h"
#include <math.h>
#include <string.h>
#include <stdio.h>

#define phi(S)         ssGetSFcnParam(S,0)   /* Input switch clock phase */
#define ts(S)          ssGetSFcnParam(S,1)   /* Sampling time */
#define c(S)           ssGetSFcnParam(S,2)   /* Capacitor values and non-linear coefficients */
#define var(S)         ssGetSFcnParam(S,3)   /* Integrator weight's variance, eq. vrms input and temperature */
#define ops(S)         ssGetSFcnParam(S,4)   /* Output swing limits */
#define ao(S)          ssGetSFcnParam(S,5)   /* OTA Finite DC gain, transconductance and maximum output current */
#define setnoise(S)    ssGetSFcnParam(S,6)   /* Flag that activates the effect of thermal noise   */
#define avnl(S)        ssGetSFcnParam(S,7)   /* Non-linear coefficients of the finite DC gain */
#define cp(S)          ssGetSFcnParam(S,8)   /* OTA parasitic capacitances and load capacitances */
#define settlingon(S)  ssGetSFcnParam(S,9)   /* Flag that activates incomplete settling model */
#define setin(S)       ssGetSFcnParam(S,10)  /* Integrator identifier used in the incomplete settling model */
#define idown(S)       ssGetSFcnParam(S,11)  /* Integrator identifier */
#define idnext(S)      ssGetSFcnParam(S,12)  /* Identifier of the Integrator connected at the Integrator output */
#define setloads(S)    ssGetSFcnParam(S,13)  /* Activate the effect of additional load capacitances in different clock phases */
#define CLOAD_S(S)     ssGetSFcnParam(S,14)  /* Additional load capacitance in the sampling phase */
#define CLOAD_I(S)     ssGetSFcnParam(S,15)  /* Additional load capacitance in the integration phase */
#define setphases(S)    ssGetSFcnParam(S,16) /* Flag that activates the reduction of the clock phase switch-on time */
#define phases_reduction(S)    ssGetSFcnParam(S,17) /* Duty cycle, reduction of the integration and sampling time */
#define BW(S)    ssGetSFcnParam(S,18)		/*  SIGNAL BANDWIDTH */
#define numid(S)    ssGetSFcnParam(S,19)	/* Integrator identification number, i.e. int1, int2, int3... */
#define NPARAM 20

static void mdlInitializeSizes(SimStruct *S)
{
    
    ssSetNumSFcnParams(S, NPARAM);
    if (ssGetNumSFcnParams(S)  != NPARAM)
       return;
    ssSetNumContStates(S, 0);
    ssSetNumDiscStates(S, 0);

    if (!ssSetNumInputPorts(S, 4)) return;
    ssSetInputPortWidth (S, 0, 1);
    ssSetInputPortWidth (S, 1, 1);
    ssSetInputPortWidth (S, 2, 1);
    ssSetInputPortWidth (S, 3, DYNAMICALLY_SIZED); /* Input port where the "From" block is connected */
    ssSetInputPortDirectFeedThrough(S, 0, 1);
    ssSetInputPortDirectFeedThrough(S, 1, 1);
    ssSetInputPortDirectFeedThrough(S, 2, 1);
    ssSetInputPortDirectFeedThrough(S, 3, 1);
    
    if (!ssSetNumOutputPorts(S, 2)) return;
    ssSetOutputPortWidth (S, 0, 1);
    ssSetOutputPortWidth (S, 1, 1);

    ssSetNumSampleTimes(S, 1);
    ssSetNumRWork(S, 9);
    ssSetNumIWork(S, 1);
    ssSetNumPWork(S, 0);
    ssSetNumModes(S, 0);
    ssSetNumNonsampledZCs(S, 0);
    ssSetOptions(S, SS_OPTION_EXCEPTION_FREE_CODE);
		                
}

static void mdlInitializeSampleTimes(SimStruct *S) /* Clock-phase definition */
{
    double TS;
   
    TS = *mxGetPr(ts(S));
    ssSetSampleTime(S, 0, TS/2);
    ssSetOffsetTime(S, 0, 0);
}

static void mdlStart(real_T *x0,SimStruct *S)  /*Initial conditions */
{
    real_T *work = ssGetRWork(S);
    int_T *Iwork = ssGetIWork(S);
    
    work[0]=0;
    work[1]=0;
    work[2]=0;
    work[3]=0;
    work[4]=0;
    work[5]=0;
    work[6]=0;
    work[7]=0;
    work[8]=1000;
  
}  

static void mdlOutputs(SimStruct *S, int_T tid)
{
	/* Declaration of model variables and parameters */ 
	
     double U1, U2, PHI, cont, VC1, VC2, VC1ANT, C1, C2, ITER, AV, AVNL1,AVNL2, AVNL3, AVNL4, AVANT, AV_OLD, AVNEW, AVVAR, VO2, VO3, VO4, temp,SETIN;
     double CNL1, CNL2, CSUM; 
     double MEAN, VARIANCE, SEMILLA, zn, seed, noise_int, noise_int2, peso_final;
     double UNOISE, GM, INPSD, CP1, CP2, CLOAD, TEMP, RON;
     double RNON;
     double GMSEFF, GMIEFF; 
     double CLOAD_S, CLOAD_I; 
     double K, TS, FS, SIN_PHI1, SIN_PHI2, SOP, SIN_OP, DELTANOISE, VNOISE;
     double BWn_SPHI2,BWn_OP,SS, z1I, p1I, p2;
     double TOI, TOS, SIGNO, IO, CP, VAN, VAN1MED, VAINII, VAINIS, CEQI, CEQS, CPRIMA;
     double OPSPOS, OPSNEG;
     double SETTLINGON;
     double setloads; 
     double setnoise; 
     double setphases; 
     double duty_cycle, int_red, samp_red;
     double TINT, TSAMP;
     double *p, *phases_reduction;
     int a,m,zno,i,N;
     double numid;
     double GBSEFF, GBIEFF; 
     double SRSEFF, SRIEFF; 
	 double STOTAL_NOISE;
     double BW, SIN_PHI1_log, SIN_PHI2_log, SIN_OP_log, STOTAL_NOISE_log;
	 int step_cont;
	 double	Filter_factor, OSR, L;
	 double pi=3.1416;
	 double STOTAL_NOISE_filtered, STOTAL_NOISE_filtered_log;
     InputRealPtrsType uPtrs1 = ssGetInputPortRealSignalPtrs(S,0);
     InputRealPtrsType uPtrs2 = ssGetInputPortRealSignalPtrs(S,1);
     InputRealPtrsType uPtrs3 = ssGetInputPortRealSignalPtrs(S,2);
     InputRealPtrsType uPtrs4 = ssGetInputPortRealSignalPtrs(S,3); 
     real_T *y = ssGetOutputPortRealSignal(S,0);
     real_T *y2 = ssGetOutputPortRealSignal(S,1);
     real_T *work = ssGetRWork(S);
     int_T *Iwork = ssGetIWork(S);
	
	/* DEFINITION OF MODEL PARAMETERS */
     
     PHI = *mxGetPr(phi(S)); /* Input switch clock phase */
     setloads = *mxGetPr(setloads(S)); /* Additional Load capacitances during the sampling (S) and integration phase (I) */
     if (setloads)
     {
       CLOAD_S = *mxGetPr(CLOAD_S(S));
       CLOAD_I = *mxGetPr(CLOAD_I(S));
     }
     else
     {
       CLOAD_S = 0;
       CLOAD_I = 0;
     }
     
     setphases = *mxGetPr(setphases(S));  /* Duration of Sampling/Integration clock-phase */
     if (setphases==1)
     {
       phases_reduction = mxGetPr(phases_reduction(S));
       duty_cycle=phases_reduction[0];
       int_red=phases_reduction[1];
       samp_red=phases_reduction[2];
     }  
     else
     {
       duty_cycle=50;
       int_red=0;
       samp_red=0;
     }
	
     p = mxGetPr(c(S)); /* Capacitor values and non-linear coefficients */
     C1 = p[0];
     C2 = p[1];
     CNL1 = p[2];
     CNL2 = p[3];     
     p = mxGetPr(ao(S)); /* OTA Finite DC gain (AV), transconductance (GM) and maximum output current (IO) */
     AV = p[0];
     GM = p[1];
     IO = p[2];
     p = mxGetPr(avnl(S)); /* Non-linear coefficients of the finite DC gain */
     AVNL1 = p[0];
     AVNL2 = p[1];
     AVNL3 = p[2];
     AVNL4 = p[3];
     p = mxGetPr(var(S)); /* Integrator weight's variance, eq. vrms input and temperature */
     VARIANCE = p[0];
     INPSD = p[1];
     TEMP = p[2];
     RON = p[3];
     p = mxGetPr(cp(S)); /* OTA Input parasitic capacitances and load capacitance */
     CP1 = p[0];
     CP2 = p[1];
     CLOAD = p[2];
     TS = *mxGetPr(ts(S)); /* Sampling time */
     FS = 1/TS; /* Sampling frequency */
     p = mxGetPr(ops(S)); /* Output swing limits */
     OPSPOS = p[0];
     OPSNEG = p[1];
     SETTLINGON = *mxGetPr(settlingon(S)); /* Settling flag that activates incomplete settling model */
     SETIN = *mxGetPr(setin(S)); /* Integrator identifier to be used in the incomplete settling error computation */
    
     U1=*uPtrs1[0]; /* Value of signal connected to input branch #1 */
     U2=*uPtrs2[0]; /* Value of signal connected to input branch #2 */
     UNOISE=*uPtrs3[0]; /* Value of noise source connected at the input */
     N=*uPtrs4[0]; /* Number of input branches */
     
     cont=work[0]; /* Counter used to determine clock phase: sampling phase / integration phase */
/*	 Initial values of VC1(n), VC1(n-1)... */
     VC1=work[1];
     VC2=work[2];
     VC1ANT=work[3];
     VAN=work[4];
     VAN1MED=work[5];
     VAINII=work[6];
     VAINIS=work[7];
     AV_OLD=work[8];
/*  Reduction of clock-phase duration */
     TINT=TS*(duty_cycle/100)-int_red; /* Integration phase duration */
     TSAMP=TS*(1-duty_cycle/100)-samp_red; /*Sampling phase duration */

/*   Mismatch Parameters         */     
     MEAN = C1/C2; /* Nominal value of the integrator weight */
     SEMILLA = 1; /*Gaussian random number generator */
     a=16807;
     m=2147483647;
     seed=SEMILLA;
     zno=seed;
     zn=(a*zno)%m;
     noise_int=zn/m;
     zno=zn;
     zn=(a*zno)%m;
     seed=zn;
     noise_int2=zn/m;

/*   Calculation of the integrator weight considering capacitor mismatch */
     weigth=MEAN+(VARIANCE*sqrt(-2*log(noise_int))*cos(2*3.1416*noise_int2));
    
/*   Incomplete Settling Parameters          */	
/*   Calculation of the equivalent load capacitance in the integration 
*    phase (CEQI) and sampling phase (CEQS)*/
	 
        CP=CP2;
        CSUM=0;

	if (setloads==1)
	{
            CEQI = C1+CP+((CLOAD+CLOAD_I)*(1+((C1+CP)/C2))); 
	        CPRIMA = CP + ((CLOAD+CLOAD_I)*(1+(CP/C2)));
            if (SETIN==1)
            {
                for(i=N+1;i<=2*N;i++){
                CSUM=CSUM + (*uPtrs4[i]);}
                CEQS = CP+(CLOAD+CLOAD_S+CSUM)*(1+CP/C2);
            }
            else
            {
                CEQS= CP + ((CLOAD+CLOAD_S)*(1+(CP/C2)));
            }
	} 

	else
	{
            CEQI = C1+CP+(CLOAD*(1+((C1+CP)/C2)));
	        CPRIMA = CP + (CLOAD*(1+(CP/C2)));
            if (SETIN==1)
            {
                for(i=N+1;i<=2*N;i++){
                CSUM=CSUM + (*uPtrs4[i]);}
                CEQS = CP+(CLOAD+CSUM)*(1+CP/C2);
            }
            else
            {
                CEQS=CPRIMA;
            }
	}
    

/*   Thermal Noise Parameters and Calculation of the input referred noise voltage */
    
     setnoise = *mxGetPr(setnoise(S));
     K = 1.380658*pow(10,(-23));
    
     /*   Switches' contribution (PHI1) */ 
     SIN_PHI1 = K*TEMP/(C1*FS);
    
    /*   Switches' contribution (PHI2) */ 
     z1I=GM/CPRIMA; p1I=GM/CEQI; p2=CEQI/CPRIMA/(2*RON*C1);
     BWn_SPHI2=p1I*p2*(p1I*p2+(z1I*z1I))/(4*z1I*z1I*(p1I+p2));
     SS=2*K*TEMP*2*RON;
     SIN_PHI2 = 2*BWn_SPHI2*SS/FS;
    
    /*   Amplifier contribution */
     SOP = INPSD*INPSD;
     BWn_OP= p1I*p2/(4*(p1I+p2)); 
     SIN_OP=2*BWn_OP*SOP/FS;
	
	/* Effect of RON on thermal noise */
     
     if(RON==0)
     {
        DELTANOISE=sqrt(12*SIN_OP*FS);
     }
     else
     {
        DELTANOISE = sqrt(12*(2*SIN_PHI1+2*SIN_PHI2+SIN_OP)*FS);
     } /*   BOTH SIN_PHI ARE DOUBLED TO TAKE INTO ACCOUNT THE DIFFERENTIAL IMPLEMENTATION */
    
     if (setnoise!=1) 
     {
	DELTANOISE=0;
     }
	
     VNOISE = UNOISE*DELTANOISE;
     
     /* Code that accounts for the effect of the RON on the dynamics */ 
     RNON=RON; /*   At first, it should be given by the user */
     GMSEFF=GM*CEQS/(CEQS+(2*RNON*CSUM*GM)); /* RNON = switch-on resistance of the switches connected at the output of the integrator */
     GMIEFF=GM*CEQI/(CEQI+(2*RON*C1*GM));

	/*  Computation of the integrator model at each clock phase */
	
	/********************************************/
	/*  Input switch clocked at phi1: PHI=1     */
	/********************************************/
	
     
     if(PHI==1){
            if(cont==0){ /* SAMPLING PHASE */
              if(SETTLINGON==1){ /* Effect of Linear switch-on resistance (Ron), capacitor non-linearity and thermal noise */
                if(RON==0){
                    VC1 = -((U1*(1+(CNL1*U1/2)+(CNL2*U1*U1/3)))+VNOISE);
                }
                else{
                    VC1 = -((U1*(1+(CNL1*U1/2)+(CNL2*U1*U1/3)))+VNOISE)*(1-exp((-1)*TSAMP/(RON*C1)));
                }
                if (SETIN==1) /* Incomplete Settling - Sampling Phase */
                    {
                        VAINIS = VAN;
                        for(i=1;i<=N;i++)
                            {
                              VAINIS = VAINIS - (*uPtrs4[N+i]/CEQS)*(y[0]- (*uPtrs4[i]));
                            }
                    }
                else
                    {
                        VAINIS = VAN;
                    }        
                if(VAINIS>=0){
                    SIGNO=1;
                }
                if(VAINIS<0){
                    SIGNO=-1;
                }
                TOS = (CEQS*fabs(VAINIS)/IO)-(CEQS/GMSEFF); /* TOS and opamp input-node voltage */
                if(fabs(VAINIS)<=(IO/GMSEFF)){
                    VAN1MED = VAINIS*exp(-GMSEFF*TSAMP/(CEQS));
                }
                if((fabs(VAINIS)>(IO/GMSEFF))&&(TOS<(TSAMP))){
                    VAN1MED = (IO/GMSEFF)*SIGNO*exp(-GMSEFF*((TSAMP)-TOS)/(CEQS));
                }
                if((fabs(VAINIS)>(IO/GMSEFF))&&(TOS>=(TSAMP))){
                    VAN1MED = VAINIS-((IO/CEQS)*SIGNO*(TSAMP));
                }
                AVNEW = AV; /* Non-linear Finite DC Gain - Iterative procedure */
                ITER = 0;
                do{
                    temp = y[0]*(1+1/AVNEW)+((1+(CP/C2))*(VAN1MED-VAN)); /* opamp output-node voltage */
                    temp = temp*(1+(CNL1*temp/2)+(CNL2*temp*temp/3)); /* Capacitor non-linearity */
                    AVANT = AVNEW;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNEW = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNEW)/AVNEW);
                    ITER = ITER + 1;
                }while(AVVAR>0.01&&ITER<50);
                VC2=temp;
                work[8]=AVNEW;
                if(VC2>OPSPOS){ /* Effect of output swing */
                    VC2 = OPSPOS; /* MAXIMUM VALUE OF THE SATURATION VOLTAGE */
                }
                if(VC2<OPSNEG){
                    VC2 = OPSNEG; /* MINIMUM VALUE OF THE SATURATION VOLTAGE */
                }
                cont=cont+1;
              }
              else{
                if(RON==0){
                    VC1 = -((U1*(1+(CNL1*U1/2)+(CNL2*U1*U1/3)))+VNOISE);
                }
                else{
                    VC1 = -((U1*(1+(CNL1*U1/2)+(CNL2*U1*U1/3)))+VNOISE)*(1-exp((-1)*TSAMP/(RON*C1)));
                }
                AVNEW = AV;
                ITER = 0;
                do{
                    temp = y[0]*(1+1/AVNEW);
                    temp = temp*(1+(CNL1*temp/2)+(CNL2*temp*temp/3));                    
                    AVANT = AVNEW;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNEW = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNEW)/AVNEW);
                    ITER = ITER + 1;
                }while(AVVAR>0.01&&ITER<50);
                VC2=temp;
                work[8]=AVNEW;
                if(VC2>OPSPOS){ /*Effect of output swing*/
                    VC2 = OPSPOS;
                }
                if(VC2<OPSNEG){
                    VC2 = OPSNEG;
                }
                cont=cont+1;
              }
            }
            /* cont==1 */ /* INTEGRATION PHASE */
            else{
             if(SETTLINGON==1){
                VC1ANT = VC1;/* Effect of Linear switch-on resistance (Ron) and capacitor non-linearity */
                if(RON==0){
                    VC1 = -((U2*(1+(CNL1*U2/2)+(CNL2*U2*U2/3))));
                }
                else{
                    VC1 = -((U2*(1+(CNL1*U2/2)+(CNL2*U2*U2/3))))*(1-exp((-1)*TINT/(RON*C1)));
                }
		    /* Incomplete Settling - Integration Phase */ /* VAINII */
                  VAINII = (((1+((CLOAD+CLOAD_I)/C2))/CEQI)*C1*(-VC1+VC1ANT))+((CPRIMA/CEQI)*VAN1MED);
                /**/
                if(VAINII>0){
                    SIGNO=1;
                }
                if(VAINII<0){
                    SIGNO=-1;
                }
                TOI =(TS/2)+(CEQI*fabs(VAINII)/IO)-(CEQI/GMIEFF); /* TOI and opamp input-node voltage */
                if(fabs(VAINII)<=(IO/GMIEFF)){
                    VAN = VAINII*exp(-GMIEFF*TINT/(CEQI));
                }
                if((fabs(VAINII)>(IO/GMIEFF))&&(TOI<(2*TINT))){
                    VAN = (IO/GMIEFF)*SIGNO*exp(-GMIEFF*((2*TINT)-TOI)/(CEQI));
                }
                if((fabs(VAINII)>(IO/GMIEFF))&&(TOI>=(2*TINT))){
                    VAN = VAINII-((IO/CEQI)*SIGNO*(TINT));
                }
                AVNEW = AV;/* Non-linear Finite DC Gain */
                ITER = 0;
                do{ /* opamp output-node voltage */
                    temp = (-(weigth)*(VC1ANT-VC1) + VC2*(1+(1+CP/C2)/AV_OLD))/(1+(1+weigth+CP/C2)/AVNEW) + (1+((CP+C1)/C2))*VAN - (1+(CP/C2))*VAN1MED;
                    temp = temp/(1+(CNL1*temp/2)+(CNL2*temp*temp/3)); /* Capacitor non-linearity */
                    if(temp>OPSPOS){ /*Effect of output swing*/
                        temp=OPSPOS;
                    }
                    if(temp<OPSNEG){
                        temp=OPSNEG;
                    }
                    AVANT = AVNEW;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNEW = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNEW)/AVNEW);
                    ITER = ITER + 1;
                    }while(AVVAR>0.01&&ITER<50);
                y[0] = temp;
                VC1 = -VC1-VAN;          
                y2[0]=VC1;
                cont=0;
              }
              else{
                VC1ANT = VC1;
                if(RON==0){
                    VC1 = -((U2*(1+(CNL1*U2/2)+(CNL2*U2*U2/3))));
                }
                else{
                    VC1 = -((U2*(1+(CNL1*U2/2)+(CNL2*U2*U2/3))))*(1-exp((-1)*TINT/(RON*C1)));
                }
                AVNEW = AV;
                ITER = 0;
                do{
                    temp = (-(weigth)*(VC1ANT-VC1) + VC2*(1+(1+CP/C2)/AV_OLD))/(1+(1+weigth+CP/C2)/AVNEW);
                    temp = temp/(1+(CNL1*temp/2)+(CNL2*temp*temp/3));
                    if(temp>OPSPOS){
                        temp=OPSPOS;
                    }
                    if(temp<OPSNEG){
                        temp=OPSNEG;
                    }
                    AVANT = AVNEW;
                    VO2 = temp*temp;
                    VO3 = VO2*temp;
                    VO4 = VO3*temp;
                    AVNEW = AV*(1+(AVNL1*temp)+(AVNL2*VO2)+(AVNL3*VO3)+(AVNL4*VO4));
                    AVVAR = fabs((AVANT-AVNEW)/AVNEW);
                    ITER = ITER + 1;
                    }while(AVVAR>0.01&&ITER<50);
                y[0] = temp;
                y2[0]=-VC1;
                cont=0;
              }
            }    
     }
	
/********************************************/
/*  Input switch clocked at phi2: PHI=2     */
/********************************************/
	
     else{
	 
	 .............. /* Here goes the complementary code to that of PHI=1 */
		 
	 }        

     work[0]=cont;
     work[1]=VC1;
     work[2]=VC2;
     work[3]=VC1ANT;
     work[4]=VAN;
     work[5]=VAN1MED;
     work[6]=VAINII;
     work[7]=VAINIS;

}

static void mdlTerminate(SimStruct *S)
{
}

#ifdef  MATLAB_MEX_FILE    
#include "simulink.c"      
#else
#include "cg_sfun.h"       
#endif


