%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Synthesis example: A 4th-order CRFF LP/BPSDM   %
%% Based on [S. Asghar et al., IEEE VLSI-SoC 2012]%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% Synthesis of NTF (synthesizeNTF)
close all; clear all;
OSR=50; % Oversampling ratio
L = 4;  % Loop-filter order
%H_inf=2.048; %% OBG (notch=0)
%H_inf=1.912; %% OBG (notch=12)
H_inf=2.2; %% OBG (notch=22)
opt =2;  % Optimum placement of NTF zeros
Nlev =5; % No. of levels in embedded quantizer
tun=0;   % Notch frequency parameter used in synthesizeNTF 
notch=22; % Notch frequency
for (i=1:48) % Synthesis of NTF for 100 notch frequencies
H(i) = synthesizeNTF(L,OSR,opt,H_inf,0.01*tun);
stf(i)=H(i); % Set STF=1 with the same poles as NTF
stf(i).z=[]; 
tun=i;
end

%% Loop-filter realization in CRFF form (realizeNTF & stuffABCD) 
p=notch+1;
H=H(p);
form = 'CRFF';
[a,g,b,c] = realizeNTF(H,form)
ABCD = stuffABCD(a,g,b,c,form);

%% Compute and plot NTF, STF (calculateTF, evalTF)
[ntf, stf] = calculateTF(ABCD);
f = linspace(0,0.48,1000);
z = exp(2i*pi*f);
magNTF = dbv(evalTF(ntf,z));
magSTF = dbv(evalTF(stf,z));
figure;
plot(f,magNTF,'b',f,magSTF,'m','LineWidth',1)
xlabel('Normalized Frequency');
ylabel('Magnitude');
legend('Noise Transfer Function','Signal TransferFunction')

%% Scale loop-filter coefficients (scaleABCD, mapABCD)
% Limits on the states extracted from simulations
%x = [1.0 1.2 1.2 1.2] % Limit on the states for notch=0 (JMR)
%x = [0.85 0.75 1.4 1.45] % Limit on the states for notch =12 (JMR)
x = [1.4 1.25 1.5 1.22] % Limit on the states for notch =22 (JMR)
Np=1e4;
[ABCDs,umax] = scaleABCD(ABCD,Nlev,0.01*(p-1),x,Nlev+5,[],Np)
[as,gs,bs,cs] = mapABCD(ABCDs,form)

%% Simulation (simulateDSM)
%fs=100e6;fi=300e3;A=1;ts=1/fs;Ts=1/fs;N=65536;i=1; sigma=0.00 %% FOR f=0
fs=100e6;fi=0.01*(p-1)*fs;A=1;ts=1/fs;Ts=1/fs;N=65536 ;i=1; sigma=0.005 %%
u=0.1*sin(2*pi*fi/(fs)*[0:N-1]);
v=simulateDSM(u,ABCDs,Nlev);
spec=fft(v.*ds_hann(N))/(N/4);
figure; plot(linspace(0,0.5,N/2+1),dbv(spec(1:N/2+1)));

%% Map loop-filter coefficient onto feasible SC implementation
R = 100; % Rounding Factor
a2 = round([as].*R)./R
g2 = round([gs].*R)./R
b2 = round([bs].*R)./R
c2 = round([cs].*R)./R

asn_f(p,:)=a2
gsn_f(p,:)= g2
bsn_f(p,:)=b2
csn_f(p,:)=c2

asn=a2
gsn= g2
bsn=b2
csn=c2

