
%% function nsga_2(pop,gen)
% is a multi-objective optimization function where the input arguments are 
% pop - Population size
% gen - Total number of generations
% ... explanation of the optimization algorithm not shown for simplicity

%% EDIT THIS SECTION TO USE IT IN YOUR OPTIMIZATION
% GENERAL PARAMETERS REQUIRED FOR NSGA OPTIMIZATION

pop=20; % Population size (pop), which is 20 in this example
gen=4;  % Total number of generations (gen), which is 10 in this example
seed=50;% Seed --> argument used for RandStream function (seed), 
stream=RandStream('mt19937ar','Seed',seed);

%% - DEFINITION OF OBJECTIVES, VARIABLES AND RANGES

M=2; % Number of objectives (M), (power consumption and SNQR in this ex.)
V=5; % Number of design/decision variables (V)

% Min/Max ranges of design variable named res_g:s
min_range(1) = 0.5;
max_range(1) = 0.9;

% Min/Max ranges of design variable named obg:
min_range(2) = 1.25;
max_range(2) = 2.0;
%Min/Max ranges of design variable named SfRes1:
min_range(3) = 2.0;
max_range(3) = 30.0;
%Min/Max ranges of design variable named SfRes2:
min_range(4) = 1.0;
max_range(4) = 12.0;
%Min/Max ranges of design variable named gmqfactor:
min_range(5) = 0.0;
max_range(5) = 0.4;

% Wait/progressing bar:
h = waitbar(0,'Please wait, starting optimization...');

%% Load SIMSIDES model
    
model='Model_Scaled_Qenh_OoBGSvsResGain.mdl';
                                                                                  
load_system(model);

initial_time=cputime;

%% Initialize the population

chromosome = initialize_variables(pop, M, V, min_range, max_range);

waitbar(0,h,sprintf('Evaluating initial population...'))

for i=1:pop
    
%% EVALUATION SECTION
% Insert here all model parameters required to simulate the model
% and evaluate the objectives(SNQR and Power in this example)
% ...missing code goes here (not shown for simplicity)

    chromosome(i,6)=-SQNRb;
    chromosome(i,7)=Powerb;
% ... 

end

%% Sort the initialized population
% Sort the population using non-domination-sort.
%...
chromosome = non_domination_sort_mod(chromosome, M, V);
save generations.txt chromosome -ASCII

% Start the evolution process
%...

waitbar(0,h,sprintf('First generation completed'))
waitbar(0,h,sprintf('Beginning evolutionary optimization...'))

for i = 1 : gen
    pool = round(pop/2);
    tour = 2;
    parent_chromosome = tournament_selection(chromosome, pool, tour);
    mu = 20;
    mum = 20;
    offspring_chromosome = ...
        genetic_operator(parent_chromosome, ...
        M, V, mu, mum, min_range, max_range);
    [offpop,~]=size(offspring_chromosome);
    
 %% EVALUATE OFFSPRING --> EDIT THIS SECTION TO BE USE IT 
 %  Model code goes here (not shown for simplicity) ...
 %  ...
    for j=1:offpop
        
        res_gain=offspring_chromosome(j,1); % variable j -->chromosome(i,j)
        obg=offspring_chromosome(j,2);
        SfRes1=offspring_chromosome(j,3);
        SfRes2=offspring_chromosome(j,4);
        gmqfactor=offspring_chromosome(j,5);
        
 %%  Missing (model) code goes here (not shown for simplicity) ...
 %  ...
        
        options = simset('SrcWorkspace','current');
        sim (model,[],options)
        
        if(length(out)<N-2)
            SQNR=NaN;
        else
            SQNR=fsnr(out,1,N,fs,fin,BW,20,20,5,2,2); %SQNR calculus
        end
        SQNRb=SQNR;
 

        if SQNRb ~= SQNRb
            SQNRb=-10e3;
            Powerb=10e3;
        end

        offspring_chromosome(j,6)=-SQNRb;
        offspring_chromosome(j,7)=Powerb;
    end
% Intermediate population
    
    [main_pop,temp] = size(chromosome);
    [offspring_pop,temp] = size(offspring_chromosome);
    clear temp
  
    intermediate_chromosome(1:main_pop,:) = chromosome;
    intermediate_chromosome(main_pop + 1 : main_pop + offspring_pop,1 : M+V) = ...
        offspring_chromosome;
    intermediate_chromosome = ...
        non_domination_sort_mod(intermediate_chromosome, M, V);
 
    chromosome = replace_chromosome(intermediate_chromosome, M, V, pop);
    save generations.txt chromosome -ASCII -append
    
    nodom=0;
    for m=1:pop
        if chromosome(m,8)==1
            nodom=nodom+1;
        end
    end
    
    if ~mod(i,100)
        clc
    end
    waitbar(i/gen,h,sprintf('Generation %d/%d',i,gen))
end
delete(h)
design_time=cputime-initial_time
%% Results
save final_pop.txt chromosome -ASCII
