%% GENERATION OF COLORED NOISE SEQUENCE EXTRACTING DATA FROM HSPICE 
%
% The PSD curve must contain a 1/f and a white component
% The PSD curve must have been obtained from 0 to fs/2
% 
clear; clf;
rand('state',0) % Return RAND to its default initial state
%                 Helps to add repeatability to the results
%N=2^18;         % Length of noise time sequences to be generated
N=2^25;          % N must be >> 1 to plot noise PSD at low frequencies
%
%% Loads and plots PSD curve obtained by electrical SPICE simulation
%
data_in=load('psd_eq_ref_1f_white_RRF.dat'); % Input data is assumed to be expressed in V^2 (PSD)
f_p=data_in(:,1);
noise_psd_p=data_in(:,2);

%
%% Estimation of 1/f and white components

% Flicker (1/f) noise
Sf=noise_psd_p(1)*f_p(1);
noise_psd_1f_p=Sf./f_p;
% Thermal (white) noise
Sw=noise_psd_p(length(f_p));
noise_psd_white_p=Sw*ones(length(f_p),1);

%
%% Generation of white noise sequence
fs=2*max(f_p);
noisew_power=Sw*fs/2;
xw=sqrt(12*noisew_power);
yw=-xw/2+xw*rand(N,1);

%% Generation of 1/f noise sequence
% [1] Generation of an initial white noise sequence
fs=2*max(f_p);
noisewi_power=fs/2;
xwi=sqrt(12*noisewi_power);
ywi=-xwi/2+xwi*rand(N,1);
% [2] Computation of its FFT
ywi_freq=fft(ywi); % MATLAB FFT (double-sided spectrum)
% [3] Filtering of the white noise power with the desired H(f)
% Equivalently, the noise % amplitude is filtered with sqrt(H(f)
freq=(fs/N:fs/N:fs)'; % Double-sided frequency vector
H2_freq=freq./freq; % To validate with H(f)=1
H2_freq=Sf./freq;
yf_freq=sqrt(H2_freq).*(ywi_freq);
yf=real(ifft(yf_freq)); % MATLAB IFFT is used for simplicity

%% Addition of white and 1/f noise sequences
y=yw+yf; % Generated noise sequence including both white and 1/f components